import 'dart:math';

import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:random/config/default_activity_settings.dart';
import 'package:random/models/settings/settings_activity.dart';
import 'package:random/models/settings/settings_global.dart';

class ParameterPainter extends CustomPainter {
  const ParameterPainter({
    required this.code,
    required this.value,
    required this.activitySettings,
    required this.globalSettings,
  });

  final String code;
  final String value;
  final ActivitySettings activitySettings;
  final GlobalSettings globalSettings;

  @override
  void paint(Canvas canvas, Size size) {
    // force square
    final double canvasSize = min(size.width, size.height);

    // content
    switch (code) {
      case DefaultActivitySettings.parameterCodeBoardSize:
        paintBoardSizeParameterItem(canvas, canvasSize);
        break;
      case DefaultActivitySettings.parameterCodeColorsCount:
        paintColorsCountParameterItem(canvas, canvasSize);
        break;
      default:
        printlog('$ParameterPainter -> unknown parameter: $code/$value');
        paintUnknownParameterItem(canvas, canvasSize);
    }
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return false;
  }

  // "unknown" parameter -> simple block with text
  void paintUnknownParameterItem(
    final Canvas canvas,
    final double size,
  ) {
    final paint = Paint();
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 3;

    final textSpan = TextSpan(
      text: '$code\n$value',
      style: const TextStyle(
        color: Colors.black,
        fontSize: 18,
        fontWeight: FontWeight.bold,
      ),
    );
    final textPainter = TextPainter(
      text: textSpan,
      textDirection: TextDirection.ltr,
      textAlign: TextAlign.center,
    );
    textPainter.layout();
    textPainter.paint(
      canvas,
      Offset(
        (size - textPainter.width) * 0.5,
        (size - textPainter.height) * 0.5,
      ),
    );
  }

  void paintBoardSizeParameterItem(
    final Canvas canvas,
    final double size,
  ) {
    int gridWidth = 1;

    switch (value) {
      case DefaultActivitySettings.boardSizeValueSmall:
        gridWidth = 2;
        break;
      case DefaultActivitySettings.boardSizeValueMedium:
        gridWidth = 3;
        break;
      case DefaultActivitySettings.boardSizeValueLarge:
        gridWidth = 4;
        break;
      case DefaultActivitySettings.boardSizeValueExtraLarge:
        gridWidth = 5;
        break;
      default:
        printlog('Wrong value for boardSize parameter value: $value');
    }

    final paint = Paint();
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 3 / 100 * size;

    // Mini grid
    final squareBackgroundColor = Colors.grey.shade200;
    final squareBorderColor = Colors.grey.shade800;

    final double cellSize = size / 7;
    final double origin = (size - gridWidth * cellSize) / 2;

    for (int row = 0; row < gridWidth; row++) {
      for (int col = 0; col < gridWidth; col++) {
        final Offset topLeft = Offset(origin + col * cellSize, origin + row * cellSize);
        final Offset bottomRight = topLeft + Offset(cellSize, cellSize);

        paint.color = squareBackgroundColor;
        paint.style = PaintingStyle.fill;
        canvas.drawRect(Rect.fromPoints(topLeft, bottomRight), paint);

        paint.color = squareBorderColor;
        paint.style = PaintingStyle.stroke;
        canvas.drawRect(Rect.fromPoints(topLeft, bottomRight), paint);
      }
    }
  }

  void paintColorsCountParameterItem(
    final Canvas canvas,
    final double size,
  ) {
    final paint = Paint();
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 3;

    // Colors preview
    const List<Offset> positions = [
      Offset(0, 0),
      Offset(1, 0),
      Offset(2, 0),
      Offset(2, 1),
      Offset(2, 2),
      Offset(1, 2),
      Offset(0, 2),
      Offset(0, 1),
    ];

    final double padding = 4 / 100 * size;
    final double margin = 3 / 100 * size;
    final double width = ((size - 2 * padding) / 3) - 2 * margin;

    final int maxValue = int.parse(value);
    for (int colorIndex = 0; colorIndex < maxValue; colorIndex++) {
      final Offset position = positions[colorIndex];

      final Offset topLeft = Offset(padding + margin + position.dx * (width + 2 * margin),
          padding + margin + position.dy * (width + 2 * margin));

      final Offset bottomRight = topLeft + Offset(width, width);

      final squareColor = Colors.pink;
      paint.color = squareColor;
      paint.style = PaintingStyle.fill;
      canvas.drawRect(Rect.fromPoints(topLeft, bottomRight), paint);

      final borderColor = squareColor.darken(20);
      paint.color = borderColor;
      paint.style = PaintingStyle.stroke;
      canvas.drawRect(Rect.fromPoints(topLeft, bottomRight), paint);
    }

    // centered text value
    final textSpan = TextSpan(
      text: value.toString(),
      style: TextStyle(
        color: Colors.black,
        fontSize: size / 4,
        fontWeight: FontWeight.bold,
      ),
    );
    final textPainter = TextPainter(
      text: textSpan,
      textDirection: TextDirection.ltr,
      textAlign: TextAlign.center,
    );
    textPainter.layout();
    textPainter.paint(
      canvas,
      Offset(
        (size - textPainter.width) * 0.5,
        (size - textPainter.height) * 0.5,
      ),
    );
  }
}
